@@ -1,6 +1,5 @@
 import openai
 import pytest
-from spyre_util import get_spyre_backend_list, get_spyre_model_list
 
 
 def _check_result(client, model, max_tokens=8, temperature=0.0, n=1) -> None:
@@ -15,34 +14,10 @@ def _check_result(client, model, max_tokens=8, temperature=0.0, n=1) -> None:
     assert len(completion.choices[0].text) > 0
 
 
-@pytest.mark.parametrize("model", get_spyre_model_list())
-@pytest.mark.parametrize(
-    "tp_size",
-    [
-        pytest.param(1, marks=pytest.mark.basic),
-        pytest.param(2, marks=pytest.mark.multi),
-        pytest.param(4, marks=pytest.mark.multi),
-        pytest.param(8, marks=pytest.mark.multi),
-    ],
-    ids=lambda val: f"TP({val})",
-)
-@pytest.mark.parametrize("backend", get_spyre_backend_list())
-@pytest.mark.parametrize(
-    "warmup_shape",
-    [[
-        (64, 20, 1),
-    ]],
-)
-@pytest.mark.parametrize("cb",
-                         [pytest.param(1, marks=pytest.mark.cb, id="cb"), 0])
-@pytest.mark.parametrize("max_num_seqs", [4],
-                         ids=lambda val: f"max_num_seqs({val})")
-@pytest.mark.parametrize("max_model_len", [256],
-                         ids=lambda val: f"max_model_len({val})")
 def test_openai_serving(
     remote_openai_server,
     model,
-    warmup_shape,
+    warmup_shapes,
     backend,
     tp_size,
     cb,
