@@ -4,22 +4,19 @@ Run `python -m pytest tests/e2e/test_spyre_max_prompt_length.py`.
 """
 
 import pytest
-from spyre_util import (create_text_prompt, force_engine_shutdown,
-                        get_spyre_backend_list, get_spyre_model_list,
-                        patch_warmup_shapes)
-from vllm import LLM, SamplingParams
+from llm_cache import DecodeWarmupShapes
+from spyre_util import create_text_prompt, get_cached_llm
+from vllm import SamplingParams
 
 
-@pytest.mark.parametrize("model", get_spyre_model_list())
 @pytest.mark.parametrize(
     "warmup_shapes",
     [[(64, 20, 4)], [(64, 20, 4),
-                     (128, 20, 4)]])  # (prompt_length/new_tokens/batch_size)
-@pytest.mark.parametrize("backend", get_spyre_backend_list())
+                     (128, 20, 2)]])  # (prompt_length/new_tokens/batch_size)
 def test_max_prompt_len_and_new_tokens(model: str,
-                                       warmup_shapes: list[tuple[int, int,
-                                                                 int]],
-                                       backend: str, monkeypatch) -> None:
+                                       warmup_shapes: DecodeWarmupShapes,
+                                       backend: str, use_llm_cache,
+                                       monkeypatch) -> None:
     '''
     Simple test that for static batching:
     - prompts cannot exceed the maximum prompt length of all warmup shapes
@@ -28,12 +25,23 @@ def test_max_prompt_len_and_new_tokens(model: str,
 
     These two cases are combined to reduce the cost of starting each `LLM`
     '''
-    monkeypatch.setenv("VLLM_SPYRE_DYNAMO_BACKEND", backend)
-    patch_warmup_shapes(warmup_shapes, monkeypatch)
+    # monkeypatch.setenv("VLLM_SPYRE_DYNAMO_BACKEND", backend)
+    # patch_warmup_shapes(warmup_shapes, monkeypatch)
 
     max_prompt_length = max([t[0] for t in warmup_shapes])
     max_new_tokens = max([t[1] for t in warmup_shapes])
-    llm = LLM(model)
+    # llm = LLM(model)
+
+    llm = get_cached_llm(
+        model=model,
+        max_model_len=256,
+        tensor_parallel_size=1,
+        backend=backend,
+        monkeypatch=monkeypatch,
+        warmup_shapes=warmup_shapes,
+        max_num_seqs=None,
+        use_cb=False,
+    )
 
     # Craft a request with a prompt that is slightly too long for the warmup
     # shape
@@ -56,5 +64,3 @@ def test_max_prompt_len_and_new_tokens(model: str,
         results = llm.generate(prompts=[prompt],
                                sampling_params=sampling_params)
         assert results[0].outputs[0].text == ""
-
-    force_engine_shutdown(llm)
