@@ -2,7 +2,6 @@ import inspect
 import os
 
 import pytest
-from spyre_util import get_spyre_model_list
 
 from vllm_spyre.compat_utils import dataclass_fields
 
@@ -33,7 +32,6 @@ def test_vllm_bert_support():
 
 
 @pytest.mark.cpu
-@pytest.mark.parametrize("model", get_spyre_model_list())
 def test_model_config_task(model: str):
 
     from vllm.engine.arg_utils import EngineArgs
@@ -170,3 +168,20 @@ def test_mm_inputs():
                                "renamed mm_inputs to mm_kwargs.")
         # The compat code introduced in the PR below can now be removed:
         # https://github.com/vllm-project/vllm-spyre/pull/380
+
+
+@pytest.mark.cpu
+def test_init_builtin_logitsprocs():
+
+    import vllm.v1.sample.logits_processor
+    has_init_builtin_logitsprocs = hasattr(vllm.v1.sample.logits_processor,
+                                           "init_builtin_logitsprocs")
+
+    if VLLM_VERSION == "vLLM:main":
+        assert not has_init_builtin_logitsprocs
+    elif VLLM_VERSION == "vLLM:lowest":
+        assert has_init_builtin_logitsprocs, (
+            "The lowest supported vLLM version already"
+            "refactored init_builtin_logitsprocs.")
+        # The compat code introduced in the PR below can now be removed:
+        # https://github.com/vllm-project/vllm-spyre/pull/443
