@@ -19,6 +19,8 @@ if TYPE_CHECKING:
     # for experimentation purposes.
     VLLM_SPYRE_ENABLE_PROMPT_LOGPROBS: bool = False
     VLLM_SPYRE_UPDATE_THREAD_CONFIG: bool = True
+    VLLM_SPYRE_MAX_LOAD_PROCESSES: int = 0
+    VLLM_SPYRE_WORKER_LOG_REDIRECT_DIR: str = ""
 
 logger = init_logger(__name__)
 
@@ -115,7 +117,14 @@ environment_variables: dict[str, Callable[[], Any]] = {
     # memory usage.
     # Set to 0 to allow any number of processes
     "VLLM_SPYRE_MAX_LOAD_PROCESSES":
-    lambda: bool(int(os.getenv("VLLM_SPYRE_MAX_LOAD_PROCESSES", "0"))),
+    lambda: int(os.getenv("VLLM_SPYRE_MAX_LOAD_PROCESSES", "0")),
+
+    # If set, redirects all stdout and stderr from worker processes to files
+    # within this director. This is useful for debugging card-specific errors
+    # in multi-AIU setups, but should never be enabled in production settings.
+    # This removes all output from stdout and stderr for the worker processes.
+    "VLLM_SPYRE_WORKER_LOG_REDIRECT_DIR":
+    lambda: os.getenv("VLLM_SPYRE_WORKER_LOG_REDIRECT_DIR", ""),
 }
 # --8<-- [end:env-vars-definition]
 
